@@ -224,6 +224,10 @@ desktop_callback_check_done (DesktopCallback *desktop_callback)
         return;
     }
 
+    /* Ensure our metadata is updated before calling back. */
+    caja_desktop_update_metadata_from_mateconf(CAJA_FILE (desktop_callback->desktop_file),
+                                               "directory");
+
     /* Remove from the hash table before sending it. */
     g_hash_table_remove (desktop_callback->desktop_file->details->callbacks,
                          desktop_callback);
@@ -655,8 +659,6 @@ caja_desktop_directory_file_init (gpointer object, gpointer klass)
 
     desktop_file->details->real_dir_file = real_dir_file;
 
-    caja_desktop_update_metadata_from_mateconf (CAJA_FILE (desktop_file), "directory");
-
     g_signal_connect_object (real_dir_file, "changed",
                              G_CALLBACK (real_file_changed_callback), desktop_file, 0);
 }
